@@ -278,22 +278,33 @@ class Algorithms:
 
     def ccm(self, frame: np.ndarray, matrix: np.ndarray, clip=True) -> np.ndarray:
         """
-        Apply the given color correction matrix on the given RGB frame. If the
-        'clip' flag is True, input colors are clipped to [0, 1] to avoid "pink
-        sky" artifacts caused by the combination of clipped highlights and
-        less-than-1.0 coefficients in the CCM. No attempt is made at gamut
-        mapping or highlight recovery. If matrix is None, the frame is returned
-        untouched.
+        Apply the given global or per-pixel color correction matrix/-es on the
+        given RGB frame. If the 'clip' flag is True, input colors are clipped to
+        [0, 1] to avoid "pink sky" artifacts caused by the combination of clipped
+        highlights and less-than-1.0 coefficients in the CCM. No attempt is made
+        at gamut mapping or highlight recovery. If matrix is None, the frame is
+        returned untouched.
         """
         if matrix is not None:
+            assert frame.dtype in [np.float16, np.float32, np.float64], f"expected floating-point pixels, not {frame.dtype}"
+            assert frame.ndim == 3, f"expected a regular 2D frame, not shape = {frame.shape}"
+            assert matrix.ndim in [2, 4], f"CCM must be either global or per-pixel"
             if clip:
                 frame = self.clip(frame, 0, 1)
             t0 = time.time()
             matrix = matrix.astype(frame.dtype)
-            frame = frame @ matrix.T
-            with np.printoptions(formatter={'float': lambda x: f"{x:.2f}"}):
-                sums = f"with column sums {np.sum(matrix, axis=0).T}"
-                self._vprint(f"{_elapsed(t0)} - applying CCM {sums}: range = {self._minmax(frame)}")
+            if matrix.ndim == 2:
+                assert matrix.shape[1] == frame.shape[2], f"invalid global CCM: {matrix.shape}"
+                frame = np.einsum("ij,hwj->hwi", matrix, frame)  # (3, 3) x (H, W, 3) => (H, W, 3)
+                with np.printoptions(formatter={'float': lambda x: f"{x:.2f}"}):
+                    sums = f"with column sums {np.sum(matrix, axis=0).T}"
+                    self._vprint(f"{_elapsed(t0)} - applying global CCM {sums}: range = {self._minmax(frame)}")
+            else:
+                assert matrix.shape[:3] == frame.shape, f"invalid per-pixel CCM grid: {matrix.shape}"
+                assert matrix.shape[3] == frame.shape[2], f"invalid per-pixel CCM grid: {matrix.shape}"
+                frame = np.einsum("hwij,hwj->hwi", matrix, frame)  # (H, W, 3, 3) x (H, W, 3) => (H, W, 3)
+                with np.printoptions(formatter={'float': lambda x: f"{x:.2f}"}):
+                    self._vprint(f"{_elapsed(t0)} - applying per-pixel CCMs: range = {self._minmax(frame)}")
         return frame
 
     def gamut(self, frame: np.ndarray, mode: str = "ACES", p=1.2) -> np.ndarray:
