@@ -39,6 +39,20 @@ class RegressionTest(unittest.TestCase):
         raw = rawpipe.verbose.quantize(raw, maxval, expected.dtype)
         self.assertEqual(np.count_nonzero(expected - raw), 0)
 
+    def test_per_pixel_ccm(self):
+        img = np.arange(4 * 3).reshape(2, 2, 3).astype(np.float64)
+        ccm = np.random.random((2, 2, 3, 3))
+        expected = np.zeros((2, 2, 3, 1))
+        expected[0, 0] = ccm[0, 0] @ img[0, [0]].T
+        expected[0, 1] = ccm[0, 1] @ img[0, [1]].T
+        expected[1, 0] = ccm[1, 0] @ img[1, [0]].T
+        expected[1, 1] = ccm[1, 1] @ img[1, [1]].T
+        expected = expected[:, :, :, 0]  # (2, 2, 3, 1) => (2, 2, 3)
+        result1 = np.einsum("hwij,hwj->hwi", ccm, img)
+        result2 = rawpipe.ccm(img, ccm, clip=False)
+        np.testing.assert_allclose(expected, result1)
+        np.testing.assert_allclose(expected, result2)
+
     def test_bayer_wb(self):
         expected, maxval = imgio.imread(os.path.join(thisdir, "expected.ppm"))
         raw = np.fromfile(os.path.join(thisdir, "input.raw"), dtype=np.uint16)
@@ -78,10 +92,11 @@ class RegressionTest(unittest.TestCase):
             res19 = rawpipe.saturate(rgb_src, lambda v: v ** 0.7)
             res20 = rawpipe.gamma(raw_src)
             res21 = rawpipe.gamma(rgb_src)
+            res22 = rawpipe.ccm(rgb_src, np.random.random((28, 12, 3, 3)))
             results = [res01, res02, res03, res04, res05, res06,
                        res07, res10, res11, res12, res13, res14,
                        res15, res16, res17, res18, res19, res20,
-                       res21]
+                       res21, res22]
             for res in results:
                 self.assertEqual(res.dtype, dt)
             self.assertEqual(res08.dtype, np.float32)  # linearize()
